@@ -23,12 +23,12 @@ IGNORE_DEPS = {
     "isort",
     "mypy",
     "pip",
-    "setuptools",
     "pytest",
     "pytest-cov",
     "pytest-env",
-    "pytest-xdist",
     "pytest-timeout",
+    "pytest-xdist",
+    "setuptools",
 }
 
 POLICY_MONTHS = {"python": 30, "numpy": 18}
@@ -133,7 +133,7 @@ def process_pkg(
     - publication date of version suggested by policy (YYYY-MM-DD)
     - status ("<", "=", "> (!)")
     """
-    print("Analyzing %s..." % pkg)
+    print(f"Analyzing {pkg}...")
     versions = query_conda(pkg)
 
     try:
@@ -162,11 +162,11 @@ def process_pkg(
         status = "<"
     elif (req_major, req_minor) > (policy_major, policy_minor):
         status = "> (!)"
-        delta = relativedelta(datetime.now(), policy_published_actual).normalized()
+        delta = relativedelta(datetime.now(), req_published).normalized()
         n_months = delta.years * 12 + delta.months
         warning(
-            f"Package is too new: {pkg}={policy_major}.{policy_minor} was "
-            f"published on {versions[policy_major, policy_minor]:%Y-%m-%d} "
+            f"Package is too new: {pkg}={req_major}.{req_minor} was "
+            f"published on {req_published:%Y-%m-%d} "
             f"which was {n_months} months ago (policy is {policy_months} months)"
         )
     else:
